/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.partitionmapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding.CREATE_STARTEGY;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.Creates;
import org.fortiss.af3.exploration.alg.dse.module.CreatorModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.random.RandomExploration;
import org.opt4j.core.problem.Creator;

import com.google.inject.Provides;

/**
 * {@link Creator} for {@link PartitionMappingEncoding}s.
 * 
 * @author diewald
 */
public class PartitionMappingCreator extends CreatorModule<PartitionMappingEncoding> {

	/**
	 * Creates an initial {@link PartitionMappingEncoding} using strategies using the minimal,
	 * maximal, or a random number of created {@link Partition}s w.r.t. the number of
	 * {@link ITaskAdapter}s
	 */
	@Provides
	@Creates
	public PartitionMappingEncoding create(Collection<IExecutionUnitAdapter<?>> execUnits,
			Collection<ITaskAdapter<?>> tasks) {
		int numTasks = tasks.size();
		// "Probability distribution" for the strategy selection: |1|numTasks - 2|1| ==> numTasks
		int strategySelIdx = RandomExploration.getInstance().nextInt(numTasks);
		if(strategySelIdx == 0) {
			return new PartitionMappingEncoding(execUnits, numTasks, CREATE_STARTEGY.MIN_SET);
		} else if(strategySelIdx == numTasks) {
			return new PartitionMappingEncoding(execUnits, numTasks, CREATE_STARTEGY.MAX_SET);
		}
		return new PartitionMappingEncoding(execUnits, numTasks, CREATE_STARTEGY.RANDOM_SET);
	}

	/** {@inheritDoc} */
	@Override
	public PartitionMappingEncoding create() {
		// TODO Auto-generated method stub
		return null;
	}
}
