/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.TaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;

/**
 * Problem encoding consisting of redundant {@link TaskMappingEntry}s with fail-silent assumption.
 * 
 * @author huang
 */
public class FailSilentTaskMappingEncoding
		extends InstantiatedTaskMappingEncoding<FailSilentTaskMappingEntry> {

	/** Copy Constructor. */
	public FailSilentTaskMappingEncoding(Collection<IExecutionUnitAdapter<?>> resources,
			TaskGraphEncoding<?> tgEncoding) {
		super(resources, tgEncoding);
	}

	// TODO: Check whether the two constructors below are actually needed.
	/**
	 * Constructor that optionally initializes the encoding to a random encoding
	 * 
	 * @throws ExplorationException
	 *             if no valid allocation targets can be identified for one {@link ITaskAdapter}.
	 */
	public FailSilentTaskMappingEncoding(Collection<IExecutionUnitAdapter<?>> resources,
			AbstractTaskGraphEncoding tgEncoding, boolean initialize) throws ExplorationException {
		super(resources, tgEncoding, initialize);
	}

	/**
	 * Constructor which derives the resulting {@link FailSilentTaskMappingEncoding} from the given
	 * {@link AbstractTaskMappingEncoding}.
	 */
	public FailSilentTaskMappingEncoding(Collection<? extends IExecutionUnitAdapter<?>> resources,
			AbstractTaskGraphEncoding tgEncoding, AbstractTaskMappingEncoding abstractEncoding) {
		super(resources, tgEncoding, abstractEncoding);
	}

	/** {@inheritDoc} */
	@Override
	public FailSilentTaskMappingEntry createEntry(ITaskAdapter<?> deployableComponent,
			IExecutionUnitAdapter<?> deploymentTarget) {
		// Request not needed to construct a {@link FailSilentTaskMappingEncoding}
		if(deploymentTarget == null) {
			return null;
		}

		return new FailSilentTaskMappingEntry(deployableComponent, deploymentTarget);
	}

	/** {@inheritDoc} */
	@Override
	public <S extends TaskMappingEntry<ITaskAdapter<?>, IExecutionUnitAdapter<?>>>
			FailSilentTaskMappingEntry createEntry(S mapping) {
		return new FailSilentTaskMappingEntry((AbstractTaskMappingEntry)mapping);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public FailSilentTaskMappingEncoding newInstance() {
		try {
			return new FailSilentTaskMappingEncoding(null, null, true);
		} catch(ExplorationException e) {
			// Opt4J does not allow to throw Exceptions here.
			throw new RuntimeException(e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public FailSilentTaskMappingEncoding copy() {
		FailSilentTaskMappingEncoding clonedEncoding =
				new FailSilentTaskMappingEncoding(getResources(), tgEncoding);

		Map<FailSilentTaskMappingEntry, FailSilentTaskMappingEntry> copyAssocMap = new HashMap<>();
		clonedEncoding.allocationMap = copyAllocationMap(copyAssocMap);
		clonedEncoding.constraintMap = copyConstraintMap(copyAssocMap);
		clonedEncoding.getRequesters().forEach(c -> c.updateInstatiationData());

		return clonedEncoding;
	}
}
