/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.constraint;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.Opt4JDseTaskProvider;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ProvidingPhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.thread.ThreadScoped;
import org.opt4j.core.Individual;
import org.opt4j.core.IndividualSet;
import org.opt4j.core.IndividualSetListener;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;

/**
 * Instance of the {@link ConstrainedArtifactMap} for {@link Phenotype}s. Here, the artifacts are
 * {@link Phenotype} TYPES since it is required to define constraints before the corresponding
 * {@link Phenotype} is created.
 * 
 * @author diewald
 */
@ThreadScoped
public class ConstraintedPhenotypeMap extends ConstrainedArtifactMap<Class<Phenotype>> {

	/**
	 * Constructor.
	 * 
	 * @param injectorProvider
	 *            References the Opt4J {@link Injector} {@link Provider} to install the listeners.
	 */
	@Inject
	public ConstraintedPhenotypeMap(Opt4JDseTaskProvider injectorProvider) {
		IndividualSet individuals = injectorProvider.getInjector().getInstance(IndividualSet.class);
		individuals.addListener(new IndividualSetListener() {
			@SuppressWarnings("unchecked")
			@Override
			public void individualRemoved(IndividualSet collection, Individual individual) {
				Phenotype phenotype = (Phenotype)individual.getPhenotype();
				removeArtifact((Class<Phenotype>)phenotype.getClass());
				// TODO: Introduce a Composite Phenotype to be more generic.
				if(phenotype instanceof ProvidingPhenotype) {
					removePhenotypesRecursively((ProvidingPhenotype)phenotype);
				}
			}

			@Override
			public void individualAdded(IndividualSet collection, Individual individual) {
				// TODO Add initially extracted constraints here.
			}
		});
	}

	/**
	 * Removes all associations in the backend map that are found in the given
	 * {@link ProvidingPhenotype}.
	 */
	@SuppressWarnings("unchecked")
	private void removePhenotypesRecursively(ProvidingPhenotype providingPhenotype) {
		providingPhenotype.getPhenotypeMap().entrySet().forEach(e -> {
			// FIXME: Assumption that we ALWAYS have the key as a class.
			removeArtifact((Class<Phenotype>)e.getKey());
			if(e instanceof ProvidingPhenotype) {
				removePhenotypesRecursively((ProvidingPhenotype)e);
			}
		});
	}
}
