/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.constraint;

import java.util.Collection;

import org.fortiss.af3.exploration.model.ExplorationConstraint;

/**
 * Specifies the interface of constraint holders such as associative maps. Internally, artifacts are
 * related to the set of constraints that a producer or modifier has to apply.
 * 
 * @author diewald
 */
public interface IConstrained<T> {

	/**
	 * Returns the set of {@link ExplorationConstraint}s that have to be applied during construction
	 * of a given artifact type, or the modification of an artifact instance.
	 * 
	 * @param constrainedArtifact
	 *            Artifact instance or type for which to retrieve the constraints.
	 * @return Set of constraints to apply.
	 */
	Collection<ExplorationConstraint<?>> getConstraintsOf(T constrainedArtifact);

	/**
	 * Adds the given set of {@link ExplorationConstraint}s to the set set of constraints that shall
	 * be considered during the construction or modification of the given artifact instance or type.
	 * 
	 * @param constrainedArtifact
	 *            Artifact instance or type to be constrained.
	 * @param constraints
	 *            Set of additional constraints.
	 */
	void putConstraints(T constrainedArtifact, Collection<ExplorationConstraint<?>> constraints);

	/**
	 * Remove all constraints from the given artifact instance or type.
	 * 
	 * @param constrainedArtifact
	 *            Artifact instance or type to remove the constraints from.
	 */
	void removeArtifact(T constrainedArtifact);
}
