/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingCache;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Provides an {@link EncodingCache} for {@link Genotype}s that collects all sub {@link Genotype}s
 * of a single Individual in the create phase.
 * 
 * @author diewald
 */
public class GenotypeCache extends EncodingCache<Genotype> {

	/**
	 * Provides a {@link Genotype} cache per DSE execution. The creator is responsible to clear the
	 * internal map before using it.
	 */
	@Provides
	@Singleton
	public GenotypeCache getEncodingCache() {
		// Note: We are using the Singleton Scope here to obtain the same instance for each creator.
		// Singletons are destroyed when the injector is gone, so we do not run into problems in
		// subsequent executions.
		// Creators are not parallelized such that we do not need the ThreadScope here.
		return new GenotypeCache();
	}
}
