/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.IEvaluatorModule;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.Objective;
import org.opt4j.core.Value;

import com.google.inject.Injector;

/**
 * Interface for evaluators of {@link Phenotype}s. It allows to construct evaluators in a
 * composable fashion where evaluators with sub-evaluators perform merge operations. This structure
 * is intended to launch evaluators based on the available input data.
 * 
 * @author diewald
 */
public interface ICompositeEvaluator<R> {

	/**
	 * Performs the actual evaluation, or execution of sub-evaluators and the merge of their
	 * results.
	 */
	Value<R> evaluate();

	/**
	 * Returns the Opt4J {@link Objective} that is evaluated by this evaluator. It is the equivalent
	 * of the {@link ExplorationTarget}.
	 */
	Objective getObjective();

	/** Returns the set of sub-evaluators of this {@link IEvaluatorModule}. */
	Set<IEvaluatorModule<R, ?, ?>> getAllSubEvaluators();

	/** Passes the {@link Injector} responsible for the injection phase. */
	void setEvaluationInjectorIfAbsent(Injector evaluationInjector);
}
