/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.comparch;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.component.model.ComponentArchitecture;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationStrategy;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.base.TransformationStrategyBase;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.deployment.ExplorationSolutionToDeployment;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEncoding;
import org.fortiss.af3.exploration.model.solutions.SingleExplorationSolution;
import org.fortiss.tooling.kernel.model.INamedElement;

/**
 * {@link ITransformationStrategy} for {@link ComponentArchitecture}s based on an
 * {@link SingleExplorationSolution} explored by the DSE.
 * 
 * @author diewald
 */
public class ComponentArchitectureTransformer
		extends TransformationStrategyBase<ComponentArchitecture> {

	/**
	 * Returns the list of elements that were instantiated in this generation/transformation step.
	 */
	public Collection<EObject> getInstantiatedElements() {
		return instantiatedElements;
	}

	/** {@inheritDoc} */
	@Override
	public Class<ComponentArchitecture> getOutputType() {
		return ComponentArchitecture.class;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(ComponentArchitecture.class);
		inputTypes.add(IMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<? extends ITransformationModule<?>>> getParentModules() {
		Collection<Class<? extends ITransformationModule<?>>> parentModules = new HashSet<>();
		parentModules.add(ExplorationSolutionToDeployment.class);
		return parentModules;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isOptional() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public TransformationState getActivationStage() {
		return TransformationState.INITIALIZING;
	}

	/** Sets the name of the transformed model. */
	@Override
	protected void doSetModelName() {
		String name = transformedModelName;
		if(launchedAsSubModule) {
			name = "Component Architecture - " + name;
		}

		setName(name, transformedModel);
		setName(name, transformedModel.getTopComponent());
	}

	/** Sets the name of the given Element without triggering a load of notifications. */
	private void setName(String name, INamedElement element) {
		boolean notificationsEnabled = element.eDeliver();
		if(notificationsEnabled) {
			element.eSetDeliver(false);
		}
		element.setName(name);
		element.eSetDeliver(true);
	}
}
