/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;
import static org.fortiss.af3.partition.util.PartitionModelElementFactory.createPartition;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.exploration.alg.service.module.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;

/**
 * Transforms a {@link PartitionMappingEncoding} to a {@link SuperSet} of AF3 Partitions.
 * 
 * @author diewald
 */
public class PartitionTransformation
		extends SuperSetTransformationBase<org.fortiss.af3.partition.model.Partition> {
	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject>
			SuperSet<org.fortiss.af3.partition.model.Partition> transform(TransformationState state,
					TransformationContext context, ExplorationTransformationInputs inputs,
					SubClassToInstanceMap<N> transformedModels, EObject outputModel) {
		PartitionMappingEncoding pmEncoding = inputs.getEncoding(PartitionMappingEncoding.class);

		SuperSet<org.fortiss.af3.partition.model.Partition> partitions =
				createSuperSet("Partitions", org.fortiss.af3.partition.model.Partition.class);
		int i = 0;
		for(Partition internalPartition : pmEncoding.getRequesters()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					createPartition("Partition_" + i++);
			encEntryModelElemAssoc.put(internalPartition,
					org.fortiss.af3.partition.model.Partition.class, af3Partition, Partition.class);
			partitions.getEntries().add(af3Partition);
		}
		return partitions;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(PartitionMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<org.fortiss.af3.partition.model.Partition> getOutputType() {
		return org.fortiss.af3.partition.model.Partition.class;
	}
}
