/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.mappers;

import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.task.model.Task;

/**
 * A mapper that associates input {@link Task}s with their {@link ITaskAdapter} abstraction used in
 * the D3SE.
 * 
 * @author diewald
 */
public class TaskMapper<T> {

	/** Internal association map. */
	private Map<T, ITaskAdapter<T>> taskMap;

	/** Constructor. */
	public TaskMapper(Map<T, ITaskAdapter<T>> taskMap) {
		this.taskMap = taskMap;
	}

	/** Returns the {@link ITaskAdapter} corresponding to the given {@code task}. */
	public ITaskAdapter<T> getAdapter(T task) {
		return taskMap.get(task);
	}
}
