/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.exception;

import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.model.INamedElement;

/**
 * Exception to be thrown whenever an invalid input model has been recognized.
 * 
 * @author diewald
 */
public class InvalidModelException extends Exception {
	/** Constructor. */
	public InvalidModelException(IModelElement element, String msg) {
		// @CodeFormatterOff
		super("One of the input models is incorrectly specified for the DSE.\n" +
				"The incorrectly specified element is " +
				((element instanceof INamedElement) ? ((INamedElement)element).getName() :
					element.toString()) +
				"(" + ((element instanceof INamedElement) ? ((INamedElement)element).getId() : 
					"<no ID>") +
				"). Details:\n" + msg);
		// @CodeFormatterOn
	}

	/** Constructor that stacks an additional {@link Exception}. */
	public InvalidModelException(IModelElement element, String msg, Exception e) {
		// @CodeFormatterOff
		super("One of the input models is incorrectly specified for the DSE.\n" +
				"The incorrectly specified element is " +
				((element instanceof INamedElement) ? ((INamedElement)element).getName() :
					element.toString()) +
				"(" + ((element instanceof INamedElement) ? ((INamedElement)element).getId() : 
					"<no ID>") +
				"). Details:\n" + msg, e);
		// @CodeFormatterOn
	}

	/** Exception for missing annotations in the input models. */
	public InvalidModelException(IModelElement element,
			Class<? extends IAnnotatedSpecification> annotationClass) {
		// @CodeFormatterOff
		super("One of the input models is incorrectly specified for the DSE.\n" + "The element " +
				((element instanceof INamedElement) ? ((INamedElement)element).getName() :
					element.toString()) + 
				". is missing the annotation " + annotationClass.getSimpleName() + ".");
		// @CodeFormatterOn
	}
}
