/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module;

import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.service.IExplorationExtension;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

import com.google.inject.Injector;

/**
 * Module to transform specific elements of {@link ExplorationSpecification}s into another set of
 * elements. A parent {@link IExpSpecTransformationModule} is required that passes the requested
 * elements to the {@link IExpSpecElementTransformator}s.
 * 
 * @author diewald
 */
public interface IExpSpecElementTransformator<I, O> extends IExplorationExtension {
	/** Returns the expected input element type. */
	Class<I> getInputType();

	/** Returns the output element type. */
	Class<O> getOutputType();

	/**
	 * Transforms the {@link ExplorationTarget}s of a DSE run. Examples for this preprocessing
	 * includes a transformation into {@link InternalConstraint}s, or DSML processing.
	 * 
	 * @param mainInjector
	 *            {@link Injector} providing access to the input models.
	 * @param inputElement
	 *            single element {@code I} that shall be transformed into an element of type
	 *            {@code O}.
	 * @return Possibly empty collection of transformed {@link ExplorationConstraint}s.
	 * @throws ExplorationException
	 *             If the transformation fails for any reason.
	 */
	O transform(Injector mainInjector, I inputElement) throws ExplorationException;
}
