/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.module;

import java.util.Map;

import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.service.IExpSpecTransformationService;
import org.fortiss.af3.exploration.alg.service.IExplorationExtension;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.moea.model.constraints.InternalConstraint;

import com.google.inject.Injector;

/**
 * Module that defines transformations of the given input {@link ExplorationSpecification} that are
 * executed before the D3SE is launched. They have to be registered with the
 * {@link IExpSpecTransformationService} or other {@link IExpSpecTransformationModule}s
 * (composition) in order to get executed.
 * 
 * @author diewald
 */
public interface IExpSpecTransformationModule<T> extends IExplorationExtension {

	/**
	 * Transforms the {@link ExplorationTarget}s of a DSE run. Examples for this preprocessing
	 * includes a transformation into {@link InternalConstraint}s, or DSML processing.
	 * 
	 * @param mainInjector
	 *            {@link Injector} providing access to the input models.
	 * @param inputExpTargets
	 *            map of input to transformed constraints.
	 * @return Possibly empty collection of transformed {@link ExplorationConstraint}s.
	 * @throws ExplorationException
	 *             If the transformation fails for any reason.
	 */
	Map<ExplorationTarget<?>, ExplorationTarget<?>> transform(Injector mainInjector,
			Map<ExplorationTarget<?>, ExplorationTarget<?>> inputExpTargets)
			throws ExplorationException;

	/**
	 * Registers the given module {@code T} (e.g., an {@link IExpSpecTransformationModule}) with
	 * this module/service.
	 * 
	 * @param module
	 *            to register.
	 */
	void registerModule(T module);
}
